! Stitch nest
!
! Combines velocity fields from a coarse model and n nests at one level finer to
! update the surface positions and temperatures in the coarse model
!
! dwhipp - 06/11

program stitch_nest

use definitions
use threads

implicit none

include 'mpif.h'

! Variable type declaration
character (len=4) :: cistep,cnnest

double precision :: current_time,inc,step,total,xss,yss,zss
double precision :: xmin,xmax,ymin,ymax,zmin,zmax,x0,y0,z0,dxyz,xnew,ynew,znew
double precision :: xelem(8),yelem(8),zelem(8),lsfelem(8)

integer :: err,i,ierr,iproc,istep,iter,j,k,l,leaf,level,loc,material0,nnest
integer :: nproc

logical :: pt_found_in_elem

type (box),dimension(:),allocatable :: boxes
type (cloud),dimension(:),allocatable :: ss_cl
type (cloud) :: ls_cl
type (cross_section),dimension(:),allocatable :: sections
type (face),dimension(6) :: cube_faces
type (material),dimension(:),allocatable :: mat
type (nest_info),dimension(:),allocatable :: nest
type (octreelsf) :: olsf
type (octreesolve),dimension(:),allocatable :: ss_osolve
type (octreesolve) :: ls_osolve
type (octreev),dimension(:),allocatable :: ss_ov
type (octreev) ls_ov
type (parameters) :: params
type (sheet),dimension(:,:),allocatable :: ss_surf
type (sheet),dimension(:),allocatable :: ls_surf
type (thread) :: threadinfo
type (void),dimension(:),allocatable :: ss_vo
type (void) :: ls_vo
type (ziso) :: zi

!-------------------------------------------------------------------------------

!call show_time (total,step,inc,0,'Program started$')

call mpi_init(ierr)
call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

! Init some variables
current_time=0.d0
iter=0

! Read/echo current time step and # of nests from command line
call getarg (1,cistep)
call getarg (2,cnnest)
write (*,'(a)') 'Received command line arguments:'
write (*,'(a,a)') 'istep = ',trim(cistep)
write (*,'(a,a)') 'nnest = ',trim(cnnest)
write (*,'(a)') ''

! Convert character values to integers
read (cistep,'(i4)') istep
read (cnnest,'(i4)') nnest

! Read the coarse model input file
params%infile=trim('input_ls.txt')
call read_controlling_parameters (params,threadinfo,'main')
allocate (ls_surf (params%ns),stat=err); if (err/=0) call stop_run ('Error alloc ls_surf in main$')
allocate (nest (nnest),stat=err); if (err/=0) call stop_run ('Error alloc nest in main$')

call int_to_char (cistep,4,istep)

! This may not always be the OUT directory...need to keep it general
params%irestart=istep
params%restartfile='LSOUT/time_'//cistep//'.bin'

! Read the current coarse model restart file, define surface, cloud and ov
call read_restart (params,istep,current_time,ls_osolve,ls_ov,ls_vo,ls_surf,ls_cl,zi,nest(1))

!if (nnest > 0) params%nest=.true.
allocate (ss_osolve(nnest),stat=err); if (err/=0) call stop_run ('Error alloc ss_osolve in main$')
allocate (ss_ov(nnest),stat=err); if (err/=0) call stop_run ('Error alloc ss_ov in main$')
allocate (ss_vo(nnest),stat=err); if (err/=0) call stop_run ('Error alloc ss_vo in main$')
allocate (ss_surf(params%ns,nnest),stat=err); if (err/=0) call stop_run ('Error alloc ss_surf in main$')
allocate (ss_cl(nnest),stat=err); if (err/=0) call stop_run ('Error alloc ss_cl in main$')

! Loop through fine model(s) and read their current restart files
do i=1,nnest
  ! Read the coarse model input file
  params%infile=trim('input_ss.txt')
  call read_controlling_parameters (params,threadinfo,'main')

  ! Modify params%restartfile to read previous restart file
  params%restartfile='SSOUT/time_'//cistep//'.bin'

  call read_restart (params,istep,current_time,ss_osolve(i),ss_ov(i),ss_vo(i),ss_surf(:,i),ss_cl(i),zi,nest(i))
enddo

! Read the coarse model input file
params%infile=trim('input_ls.txt')
call read_controlling_parameters (params,threadinfo,'main')

do i=1,nnest
  xmin=nest(i)%xminls
  xmax=nest(i)%xminls+nest(i)%sselemx
  ymin=nest(i)%yminls
  ymax=nest(i)%yminls+nest(i)%sselemy
  zmin=nest(i)%zminls
  zmax=nest(i)%zminls+nest(i)%sselemz
  do j=1,params%ns
    ls_surf(j)%z=ls_surf(j)%z-0.000
    write (*,'(a,i2,a,i7,a)') 'Stitching surface ',j,' (',ls_surf(j)%nsurface,' nodes)'
    do k=1,ls_surf(j)%nsurface
      if(ls_surf(j)%x(k)<=xmax .and. ls_surf(j)%x(k)>=xmin .and. ls_surf(j)%y(k)<=ymax .and. ls_surf(j)%y(k)>=ymin .and. ls_surf(j)%z(k)<=zmax .and. ls_surf(j)%z(k)>=zmin) then
        pt_found_in_elem=.false.
        do while (pt_found_in_elem == .false.)
          call octree_find_leaf(ss_osolve(i)%octree,ss_osolve(i)%noctree,      &
                               (ls_surf(j)%x(k)-xmin)/nest(i)%sselemx,         &
                               (ls_surf(j)%y(k)-ymin)/nest(i)%sselemy,         &
                               (ls_surf(j)%z(k)-zmin)/nest(i)%sselemz,leaf,    &
                               level,loc,x0,y0,z0,dxyz)
          do l=1,8
            xelem(l)=ss_osolve(i)%x(ss_osolve(i)%icon(l,leaf))
            yelem(l)=ss_osolve(i)%y(ss_osolve(i)%icon(l,leaf))
            zelem(l)=ss_osolve(i)%z(ss_osolve(i)%icon(l,leaf))
            lsfelem(l)=ss_osolve(i)%lsf(ss_osolve(i)%icon(l,leaf),j)
          enddo
          call lsf_reg(xelem,yelem,zelem,lsfelem,(ls_surf(j)%x(k)-xmin)/nest(i)%sselemx,(ls_surf(j)%y(k)-ymin)/nest(i)%sselemy,(ls_surf(j)%z(k)-zmin)/nest(i)%sselemz,pt_found_in_elem,xnew,ynew,znew)
          ls_surf(j)%x(k)=xnew*nest(i)%sselemx+xmin
          ls_surf(j)%y(k)=ynew*nest(i)%sselemy+ymin
          ls_surf(j)%z(k)=znew*nest(i)%sselemz+zmin
        enddo
      endif
    enddo
  enddo
enddo

! Write new output file
params%nest=.false.
call write_global_output (params,istep,iter,current_time,ls_osolve,ls_ov,ls_vo,ls_surf,ls_cl,zi,nest(1),'final')

!deallocate(ls_surf,ls_surf0,ss_surf,ss_surf0)
deallocate(ls_surf)!,ss_surf)

call mpi_finalize (ierr)

! Sign off
write (*,'(a)') 'Program execution complete'
end
