!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!            GET ELEM MAT NUMBER FROM CLOUD    OCT. 2012                       |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine get_elem_mat_number_from_cloud(params,leaf_mat_bin,ematnump,matel)

use definitions

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine looks at the number of cloud particle material numbers in the   |
! current element and uses various rules to assign an elemental material number|

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

!include 'mpif.h'

type (parameters) params
integer :: leaf_mat_bin(0:params%nmat),matel,ileaves,ematnump

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer :: i,mat_maxi!,iproc,nproc,ierr
logical :: mat_max_tie

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

!call mpi_comm_size (mpi_comm_world,nproc,ierr)
!call mpi_comm_rank (mpi_comm_world,iproc,ierr)

! Look in material number bins for max value
mat_maxi = sum(maxloc(leaf_mat_bin)) - 1

mat_max_tie = .false.

do i = 0,params%nmat
  if (i .ne. mat_maxi) then
    if (leaf_mat_bin(i) .eq. leaf_mat_bin(mat_maxi)) mat_max_tie = .true.
  endif
enddo

!  if (ileaves == 1673) then
!    write (*,*) 'element: ',ileaves
!    write (*,*) 'iproc: ',iproc
!    write (*,*) 'mat_maxi: ',mat_maxi
!    write (*,*) 'leaf_mat_bin: ',leaf_mat_bin
!    write (*,*) ''
!  endif




! WE SHOULD HANDLE THE CASE WHERE AN ELEMENT HAS A TIE AND PREVIOUSLY WAS USING DIVFEM
! IF SO, ASSIGN FIRST MATERIAL IN TIE????







if (mat_max_tie .and. leaf_mat_bin(mat_maxi) > 0) then
  ! Currently, we don't have a good record between iterations/time steps of the
  ! elemental material numbers. Ideally, in cases where an element existed in
  ! the previous time step, ties would be resolved by keeping the old material
  ! number. In cases where we have a tie, but in a new element, we should do
  ! something else, but it is not obvious how to handle this situation.
  !
  ! For now, we warn the user that we have a tie and will be using the first
  ! of the 'tie' materials as the material number for this iteration/step
  !if (iproc == 0) then
    !write (*,'(a)') '*** Warning ***'
    !write (*,'(a,i7,a)') 'Element ',ileaves,' has a multiple materials with the maximum number of cloud'
    !write (*,'(a)') 'particles. DOUAR will continue, but the material number for this element will be'
    !write (*,'(a)') 'will be that of the first maximum. In the future, this conflict should be resolved.'
    !write (*,'(a)') ''
    !write (*,'(a,i7,a)') 'WARNING: Element ',ileaves,' has multiple max cloud materials. Assigning first max.'
  !endif

!  if (ileaves == 1673) then
!    write (*,*) 'element: ',ileaves
!    write (*,*) 'iproc: ',iproc
!    write (*,*) 'mat_maxi: ',mat_maxi
!    write (*,*) 'leaf_mat_bin: ',leaf_mat_bin
!    !write (*,*) ''
!  endif
  if (ematnump < 1 .or. ematnump > params%nmat) then
    !write (*,'(a,i7,a)') 'WARNING: Element ',ileaves,' has multiple max cloud materials. Assigning first max.'
    matel = mat_maxi
    !write (*,'(a,i7)') 'Assigned material number ',matel
  else
    !write (*,'(a,i7,a)') 'WARNING: Element ',ileaves,' has multiple max cloud materials. Using ematnump.'
    matel = ematnump
    !write (*,'(a,i7)') 'Assigned material number ',matel
  endif
else
  matel = mat_maxi
endif

end subroutine get_elem_mat_number_from_cloud

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
