!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!                       INITIALIZE COMPACTION    MARCH 2013                    |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine initialize_compaction(params,density_str,osolve,mat)

use definitions

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine calculates the initial density profiles, for each possible      |
! element centre location along the base of the model.  These profiles will be |
! used to set elemental densities, establishing a density domain at the start  |
! of the model run that conforms to specified compaction properties.           |
! Ongoing compaction (due to sedimentation or burial as the result of          |
! deformation) is subsequently handled by the compaction routine.              |


!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

!include 'mpif.h'

type (parameters) params
type (string) :: density_str(2**params%levelmax_oct,2**params%levelmax_oct)
! In how much detail do we need to specify density_str? We will not be
! allocating/deallocating and of the density_str entires in compaction or
! calculate_density_profile.
! Note we are passing in the full structure here.

! We also need material number for each element, and compaction parameters
! for compactible material types.


! We also need LSFs, to check if we're above/below the free surface.
! Suggest for now we consider only the free surface. This could be
! adapted to look for other LSFs, to better define transitions between
! material types.

type(octreesolve) osolve
type (material) mat(0:params%nmat)


!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

!integer :: i,mat_maxi!,iproc,nproc,ierr
!logical :: mat_max_tie

integer :: i,j,k,size_str(2)
logical,dimension(:), allocatable :: calc_comp_velo

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------


! Loop over all instances of density_str (i,j 1:2**params%levelmax_oct), for each
! string:
! 1. Call calculate_density_profile(density_str(i,j), ??)
!      - this will also need material number for each element, and compaction
!        parameters for all materials (some of which may not be compactible)


! 100414 JCA
size_str=size(density_str(1,1)%density)

allocate(calc_comp_velo(size_str(1)))




do j=1,2**params%levelmax_oct
  do i=1,2**params%levelmax_oct
  
    do k=1,size_str(1)
      calc_comp_velo(k)=.false.
    enddo
  
    call calculate_density_profile(params,density_str(i,j),osolve,mat,1,i,j,             &
                                   calc_comp_velo, 0)






    density_str(i,j)%densityp=density_str(i,j)%density





  
  
  enddo
enddo

! Either here, or in calculate_density_profile, for the case of initialize=1,
! should set desnityp=density

! 100414 JCA deallocate calc_comp_velo?



end subroutine initialize_compaction

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
