!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              REMOVE LSFS    MARCH 2013                                       |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine remove_lsfs (params,osolve,threadinfo)

use definitions
use threads

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine removes LSFs from osolve that are no longer needed after surface
! removal
!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

include 'mpif.h'

type (parameters) params
type (octreesolve) osolve
type (thread) threadinfo

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|
double precision,dimension(:,:),allocatable :: lsftemp
integer :: iproc,nproc,ierr,err,ilsf,nlsforig,lsfcount

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

! We know that the number of surfaces has changed, so we remove the unneeded LSF
! info from osolve%lsf
! Note: An alternative would be to fully deallocate osolve, ov and
! olsf and reconstruct

! First we store the original number of LSFs
nlsforig=params%nsorig

! Now we allocate a temporary array to store the old LSFs, fill it and
! deallocate osolve%lsf
allocate(lsftemp(osolve%nnode,nlsforig),stat=threadinfo%err)
if (params%debug.gt.1) call heap (threadinfo,'lsftemp','remove_lsfs',size(lsftemp),'dp',+1)
lsftemp=osolve%lsf
if (params%debug.gt.1) call heap (threadinfo,'osolve%lsf','remove_lsfs',size(osolve%lsf),'dp',-1)
deallocate (osolve%lsf)

! Now we reallocate the smaller osolve%lsf, fill it and deallocate lsftemp
allocate (osolve%lsf(osolve%nnode,osolve%nlsf),stat=threadinfo%err) 
if (params%debug.gt.1) call heap (threadinfo,'osolve%lsf', 'remove_lsfs',size(osolve%lsf),'dp',+1)
lsfcount=0
do ilsf=1,nlsforig
  if (params%surfremindex(ilsf) > 0) then
    lsfcount=lsfcount+1
    osolve%lsf(:,lsfcount)=lsftemp(:,ilsf)
  endif
enddo
if (params%debug.gt.1) call heap (threadinfo,'lsftemp','remove_lsfs',size(lsftemp),'dp',-1)
deallocate (lsftemp)

end subroutine remove_lsfs

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
