!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              REMOVE SURFACES    MARCH 2013                                   |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine remove_surfaces (params,surface,surf0flag,threadinfo)

use definitions
use threads

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine removes surfaces that are flagged for removal in the input file,
! following their use to define the material numbers on the cloud. This routine
! will only be used when the cloud is used for material properties.

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

include 'mpif.h'

type (parameters) params
type (sheet) :: surface(params%nsorig)
integer :: surf0flag
type (thread) threadinfo

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|
type (sheet),dimension(:),allocatable :: surfacetemp
integer :: i,j,iproc,nproc,ierr,err,is,isnew

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

allocate (surfacetemp(params%nsorig),stat=err); if (err.ne.0) call stop_run ('Error alloc surfacetemp in remove_surfaces$')

do is=1,params%nsorig


  !write (*,*) 'In first loop with is=',is
  !write (*,*) 'surface(is)%nsurface: ',surface(is)%nsurface
  !write (*,*) 'surface(is)%nt: ',surface(is)%nt
  !write (*,*) 'size(surface): ',size(surface)


  ! Store number of surface particles and triangles in original surfaces
  surfacetemp(is)%nsurface=surface(is)%nsurface
  if (surf0flag == 0) surfacetemp(is)%nt=surface(is)%nt


  !write (*,*) 'Stored surface particles/triangles'


  ! Allocate memory for temporary surface fields
  allocate (surfacetemp(is)%x(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%x','remove_surfaces',size(surfacetemp(is)%x),'dp',+1)
  allocate (surfacetemp(is)%y(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%y','remove_surfaces',size(surfacetemp(is)%y),'dp',+1)
  allocate (surfacetemp(is)%z(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%z','remove_surfaces',size(surfacetemp(is)%z),'dp',+1)
  allocate (surfacetemp(is)%xn(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%xn','remove_surfaces',size(surfacetemp(is)%xn),'dp',+1)
  allocate (surfacetemp(is)%yn(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%yn','remove_surfaces',size(surfacetemp(is)%yn),'dp',+1)
  allocate (surfacetemp(is)%zn(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%zn','remove_surfaces',size(surfacetemp(is)%zn),'dp',+1)
  allocate (surfacetemp(is)%r(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%r','remove_surfaces',size(surfacetemp(is)%r),'dp',+1)
  allocate (surfacetemp(is)%s(surfacetemp(is)%nsurface),stat=threadinfo%err)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%s','remove_surfaces',size(surfacetemp(is)%s),'dp',+1)
  if (surf0flag == 0) then
    allocate (surfacetemp(is)%icon(3,surfacetemp(is)%nt),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%icon','remove_surfaces',size(surfacetemp(is)%icon),'int',+1)
  endif


  !write (*,*) 'After big allocate'


  ! Copy data from original surface array to surfacetemp
  surfacetemp(is)%x=surface(is)%x
  surfacetemp(is)%y=surface(is)%y
  surfacetemp(is)%z=surface(is)%z
  surfacetemp(is)%xn=surface(is)%xn
  surfacetemp(is)%yn=surface(is)%yn
  surfacetemp(is)%zn=surface(is)%zn
  surfacetemp(is)%r=surface(is)%r
  surfacetemp(is)%s=surface(is)%s
  if (surf0flag == 0) surfacetemp(is)%icon(1:3,1:surfacetemp(is)%nt)=surface(is)%icon(1:3,1:surface(is)%nt)

  ! Remove original surface fields
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%x','remove_surfaces',size(surface(is)%x),'dp',-1)
  deallocate (surface(is)%x)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%y','remove_surfaces',size(surface(is)%y),'dp',-1)
  deallocate (surface(is)%y)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%z','remove_surfaces',size(surface(is)%z),'dp',-1)
  deallocate (surface(is)%z)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%xn','remove_surfaces',size(surface(is)%xn),'dp',-1)
  deallocate (surface(is)%xn)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%yn','remove_surfaces',size(surface(is)%yn),'dp',-1)
  deallocate (surface(is)%yn)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%zn','remove_surfaces',size(surface(is)%zn),'dp',-1)
  deallocate (surface(is)%zn)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%r','remove_surfaces',size(surface(is)%r),'dp',-1)
  deallocate (surface(is)%r)
  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%s','remove_surfaces',size(surface(is)%s),'dp',-1)
  deallocate (surface(is)%s)
  if (surf0flag == 0) then
    if (params%debug.gt.1) call heap (threadinfo,'surface(is)%icon','remove_surfaces',size(surface(is)%icon),'int',-1)
    deallocate (surface(is)%icon)
  endif
enddo

! Deallocate surface structure - Commented out for now, probably not possible
!deallocate (surface)

! Allocate new surface structure - Commented due to lack of need to reallocate
!allocate (surface(params%ns),stat=err); if (err.ne.0) call stop_run ('Error alloc surface in remove_surfaces$')

do is=1,params%nsorig
  if (params%surfremindex(is) > 0) then
    isnew=params%surfremindex(is)
    ! Store number of surface particles and triangles in new surfaces
    surface(isnew)%nsurface=surfacetemp(is)%nsurface
    if (surf0flag == 0) surface(isnew)%nt=surfacetemp(is)%nt

    ! Allocate memory for new surface fields
    allocate (surface(isnew)%x(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%x','remove_surfaces',size(surface(isnew)%x),'dp',+1)
    allocate (surface(isnew)%y(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%y','remove_surfaces',size(surface(isnew)%y),'dp',+1)
    allocate (surface(isnew)%z(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%z','remove_surfaces',size(surface(isnew)%z),'dp',+1)
    allocate (surface(isnew)%xn(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%xn','remove_surfaces',size(surface(isnew)%xn),'dp',+1)
    allocate (surface(isnew)%yn(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%yn','remove_surfaces',size(surface(isnew)%yn),'dp',+1)
    allocate (surface(isnew)%zn(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%zn','remove_surfaces',size(surface(isnew)%zn),'dp',+1)
    allocate (surface(isnew)%r(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%r','remove_surfaces',size(surface(isnew)%r),'dp',+1)
    allocate (surface(isnew)%s(surface(isnew)%nsurface),stat=threadinfo%err)
    if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%s','remove_surfaces',size(surface(isnew)%s),'dp',+1)
    if (surf0flag == 0) then
      allocate (surface(isnew)%icon(3,surface(isnew)%nt),stat=threadinfo%err)
      if (params%debug.gt.1) call heap (threadinfo,'surface(isnew)%icon','remove_surfaces',size(surface(isnew)%icon),'int',+1)
    endif

    ! Copy data from surfacetemp arrays to new surface arrays
    surface(isnew)%x=surfacetemp(is)%x
    surface(isnew)%y=surfacetemp(is)%y
    surface(isnew)%z=surfacetemp(is)%z
    surface(isnew)%xn=surfacetemp(is)%xn
    surface(isnew)%yn=surfacetemp(is)%yn
    surface(isnew)%zn=surfacetemp(is)%zn
    surface(isnew)%r=surfacetemp(is)%r
    surface(isnew)%s=surfacetemp(is)%s
    if (surf0flag == 0) surface(isnew)%icon(1:3,1:surface(isnew)%nt)=surfacetemp(is)%icon(1:3,1:surfacetemp(is)%nt)
  endif

  ! Remove temporary surface fields
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%x','remove_surfaces',size(surfacetemp(is)%x),'dp',-1)
  deallocate (surfacetemp(is)%x)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%y','remove_surfaces',size(surfacetemp(is)%y),'dp',-1)
  deallocate (surfacetemp(is)%y)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%z','remove_surfaces',size(surfacetemp(is)%z),'dp',-1)
  deallocate (surfacetemp(is)%z)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%xn','remove_surfaces',size(surfacetemp(is)%xn),'dp',-1)
  deallocate (surfacetemp(is)%xn)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%yn','remove_surfaces',size(surfacetemp(is)%yn),'dp',-1)
  deallocate (surfacetemp(is)%yn)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%zn','remove_surfaces',size(surfacetemp(is)%zn),'dp',-1)
  deallocate (surfacetemp(is)%zn)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%r','remove_surfaces',size(surfacetemp(is)%r),'dp',-1)
  deallocate (surfacetemp(is)%r)
  if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%s','remove_surfaces',size(surfacetemp(is)%s),'dp',-1)
  deallocate (surfacetemp(is)%s)
  if (surf0flag == 0) then
    if (params%debug.gt.1) call heap (threadinfo,'surfacetemp(is)%icon','remove_surfaces',size(surfacetemp(is)%icon),'int',-1)
    deallocate (surfacetemp(is)%icon)
  endif
enddo

! Deallocate temporary surface structure
deallocate (surfacetemp)

! Reduce size of unused surfaces to 1 particle and 1 triangle
do is=params%ns+1,params%nsorig
  ! Store number of surface particles and triangles in new surfaces
  surface(is)%nsurface=-1
  if (surf0flag == 0) surface(is)%nt=-1
enddo

!  ! Allocate memory for new surface fields
!  allocate (surface(is)%x(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%x','remove_surfaces',size(surface(is)%x),'dp',+1)
!  allocate (surface(is)%y(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%y','remove_surfaces',size(surface(is)%y),'dp',+1)
!  allocate (surface(is)%z(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%z','remove_surfaces',size(surface(is)%z),'dp',+1)
!  allocate (surface(is)%xn(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%xn','remove_surfaces',size(surface(is)%xn),'dp',+1)
!  allocate (surface(is)%yn(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%yn','remove_surfaces',size(surface(is)%yn),'dp',+1)
!  allocate (surface(is)%zn(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%zn','remove_surfaces',size(surface(is)%zn),'dp',+1)
!  allocate (surface(is)%r(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%r','remove_surfaces',size(surface(is)%r),'dp',+1)
!  allocate (surface(is)%s(surface(is)%nsurface),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%s','remove_surfaces',size(surface(is)%s),'dp',+1)
!  allocate (surface(is)%icon(3,surface(is)%nt),stat=threadinfo%err)
!  if (params%debug.gt.1) call heap (threadinfo,'surface(is)%icon','remove_surfaces',size(surface(is)%icon),'int',+1)
!enddo

end subroutine remove_surfaces

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
