!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!                    RESIZE DENSITY PROFILES    MARCH 2013                     |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine resize_density_profiles(params,density_str,height_max)

use definitions

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine checks that the length of each density string          |


!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

implicit none

!include 'mpif.h'

type (parameters) params
type (string) :: density_str(2**params%levelmax_oct,2**params%levelmax_oct)
double precision :: height_max



!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

!integer :: i,mat_maxi!,iproc,nproc,ierr
!logical :: mat_max_tie

integer :: nzstr_new,nzstr_old,i,j,nstswap,nstnew,err,str_size
!integer,dimension(:,:),allocatable :: str_size
double precision,dimension(:,:),allocatable :: dmemswap
logical,dimension(:,:),allocatable :: lmemswap

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------

! Added option to have variable string lengths - dwhipp 04/13
!allocate(str_size(2**params%levelmax_oct,2**params%levelmax_oct))

! Loop over all instances of density_str (i,j 1:2**params%levelmax_oct), for each
! string:
! 1. Resize as necessary, to have length = (height of free surface at j,i position
!    plus buffer)/string length increment

str_size=size(density_str(1,1)%density)
!str_height=str_size(1)*STRING INCREMENT

nzstr_old=floor(height_max/((1.d0/dble(2**params%levelmax_oct)*params%vex)/dble(params%dstrnz)))
!nstradd=nzstr-str_size(1)
nzstr_new=nzstr_old+params%dstrnz

! JANICE: Flipped order of str_size and nzstr_old, should work for all cases now
if (str_size-nzstr_old.lt.dble(params%dstrnz)/2.d0) then

  nstswap=str_size
  nstnew=nzstr_new
  
  do j=1,2**params%levelmax_oct
    do i=1,2**params%levelmax_oct
   
      !copy density_str(i,j)%density, %densityp, %compactiblep to dmemswap, lmemswap
      !deallocate density_str(i,j)%density, %densityp, %compactiblep
      !allocate density_str(i,j)%density, densityp, %compactiblep of length nzstr_new
      !copy dtemp, dptemp to first str_size(1) entries of density_str(i,j)%density, %densityp, %compactiblep

      allocate (dmemswap(2,nstswap),stat=err) ; if (err.ne.0) call stop_run ('error in alloc dmemswap in resize_density_profiles$')
      allocate (lmemswap(1,nstswap),stat=err) ; if (err.ne.0) call stop_run ('error in alloc lmemswap in resize_density_profiles$')
      dmemswap(1,:)=density_str(i,j)%density
      dmemswap(2,:)=density_str(i,j)%densityp
      lmemswap(1,:)=density_str(i,j)%compactiblep

      deallocate (density_str(i,j)%density)
      deallocate (density_str(i,j)%densityp)
      deallocate (density_str(i,j)%compactiblep)

      allocate (density_str(i,j)%density(nstnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc density_str%density in resize_density_profiles$')
      allocate (density_str(i,j)%densityp(nstnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc density_str%densityp in resize_density_profiles$')
      allocate (density_str(i,j)%compactiblep(nstnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc density_str%compactiblep in resize_density_profiles$')
      
      density_str(i,j)%density=0
      density_str(i,j)%densityp=0
      density_str(i,j)%compactiblep=.false.

      density_str(i,j)%density(1:nstswap)=dmemswap(1,:)
      density_str(i,j)%densityp(1:nstswap)=dmemswap(2,:)
      density_str(i,j)%compactiblep(1:nstswap)=lmemswap(1,:)
      
      deallocate (dmemswap)
      deallocate (lmemswap)
  
    enddo
  enddo
endif



!Add new positions in density_str


!do i=1,nzstr
  !        kinject=kinject+1
!  xinject(kinject)=minval(os%x(os%icon(:,ie))) + 1.d0/params%clnpx * (0.5d0 + (xi-1)) * 1.d0/(2.d0**lev(ie))
!enddo


end subroutine resize_density_profiles

!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
