!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              SEDIMENTATION    Jan. 2012                                            |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine sedimentation (surface,olsf,is,params,current_time,time_step)

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! this routine will need major improvement in the near future
! it is used to erode the surface "surface" according to a user supplied algorithm
! at the moment it simply "shaves" the top of the model at a given height

! surface are the surface/sheet object
! (the top free surface)
! olsf is a octreelsf object containig the geometry of the current velocity octree/object
! is is the number of the surface (0 playing a special role)
! zerosion is the level of erosion (between 0 and 1)

! Note that all surfaces are required because they can be affected by the erosion process

! for the top surface, we apply the erosion and on the octree
! we define an lsf that is simply the height of the eroded surface
! This allows us, for the other surfaces to find the height of the
! free/eroded surface at any point in space and check whether the
! vertical position of the point is above or below the free surface
! We then simply "erode" the following surfaces using this informatoin

! length_scale is the length scale in km (corresponding to the unit length of the box)
! velocity_scale is the scale for the velocity in km/Myr
! dt is the length of the dimensionless time step
! fluvial_erosion is the fluvial erosion constant
! diffusion_erosion is the diffusion erosion constant

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use definitions
!use mpi

implicit none

include 'mpif.h'
     
type (sheet) surface
type (octreelsf) olsf
type (parameters) params
integer is
double precision current_time
double precision time_step

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|
integer ierr,iproc,nproc,i,nt,nh,nohalt_hull,loc,nnn,nnlist,ntrilist,k,kp,ic,icp
integer ntmax,nhmax,npmax,nmax,nmode,nvmax,nnpnmax,int_method
double precision,dimension(:),allocatable::field,wpoints
double precision,dimension(:,:),allocatable::points,centres,pointsmin,pointsmax,ppoints
integer,dimension(:,:),allocatable::vertices,neighbour
integer,dimension(:),allocatable::hulltriangles,ln_work
logical*1,dimension(:),allocatable::lt_work_l,ln_work_l
integer,dimension(:),allocatable::vis_tlist,vis_elist,add_tlist
logical clockwise,out,nnext
double precision eps
integer dmode,err
double precision x,y,f,df(2),ddf(3),v
double precision,dimension(:,:),allocatable::work_d1,work_d2,work_d3
double precision,dimension(:,:),allocatable::work_d4,work_d5,work_d6,work_d7
integer,dimension(:),allocatable::work_i1,work_i3
integer,dimension(:,:),allocatable::work_i2
real,dimension(:),allocatable::work_r2
real,dimension(:,:),allocatable::work_r1
double precision,dimension(:),allocatable::lsf
double precision,dimension(:),allocatable::xdouar,ydouar,hdouar,hdouarp
integer nnodedouar
double precision dtdouar,href
double precision dhminfluv,dhmaxfluv,dhmindiff,dhmaxdiff
double precision zerosion
double precision length_scale,velocity_scale,dt
double precision fluvial_erosion,diffusion_erosion
integer baselevelx0,baselevelx1,baselevely0,baselevely1


double precision :: zaggrade_init, aggrade_rate, zaggrade_current, zaggrade_current2
double precision :: x_agg_start, x_agg_end, x_agg_sinus_amp, x_agg_sinus_wavelth, z_compare
double precision :: y_agg_start, y_agg_end, y_agg_sinus_amp, y_agg_sinus_wavelth
double precision :: z_prog_init, z_prog_fin, x_prog_start, x_prog_end, prog_rate_u, x_prog1
double precision :: x_prog2, x_prog_length
integer :: sedimentation_type

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

if (iproc.eq.0) write(*,*) 'Opened sedimentation with surface ',is

zerosion=params%zerosion
length_scale=params%length_scale
velocity_scale=params%velocity_scale
!dt=params%dt
fluvial_erosion=params%fluvial_erosion
diffusion_erosion=params%diffusion_erosion
baselevelx0=params%baselevelx0
baselevelx1=params%baselevelx1
baselevely0=params%baselevely0
baselevely1=params%baselevely1

sedimentation_type=params%sedimentation_type
zaggrade_init=params%zaggrade_init
x_agg_start=params%x_agg_start
x_agg_end=params%x_agg_end
y_agg_start=params%y_agg_start
y_agg_end=params%y_agg_end
x_agg_sinus_amp=params%x_agg_sinus_amp
x_agg_sinus_wavelth=params%x_agg_sinus_wavelth
y_agg_sinus_amp=params%y_agg_sinus_amp
y_agg_sinus_wavelth=params%y_agg_sinus_wavelth
aggrade_rate=params%aggrade_rate
z_prog_init=params%z_prog_init
z_prog_fin=params%z_prog_fin
x_prog_start=params%x_prog_start
x_prog_end=params%x_prog_end
prog_rate_u=params%prog_rate_u
x_prog_length=params%x_prog_length
!dt=params%dt


if (is.ne.1) then

   if (surface%spread_surface_points.gt.0) then

      allocate (points(2,surface%nsurface),stat=err); if (err.ne.0) call stop_run ('Error alloc points in erode$')
      allocate (pointsmin(2,surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc pointsmin in erode$')
      allocate (pointsmax(2,surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc pointsmax in erode$')
      allocate (ln_work(surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc ln_work in erode$')

      points=0.d0
      pointsmin=2.d0
      pointsmax=-1.d0
      ln_work=0
      do i=1,surface%nt
         do k=1,3
            kp=mod(k,3)+1
            ic=surface%icon(k,i)
            icp=surface%icon(kp,i)
            points(1,ic)=points(1,ic)+surface%x(icp)
            points(2,ic)=points(2,ic)+surface%y(icp)
            pointsmin(1,ic)=min(pointsmin(1,ic),surface%x(icp))
            pointsmax(1,ic)=max(pointsmax(1,ic),surface%x(icp))
            pointsmin(2,ic)=min(pointsmin(2,ic),surface%y(icp))
            pointsmax(2,ic)=max(pointsmax(2,ic),surface%y(icp))
            ln_work(ic)=ln_work(ic)+1
         enddo
      enddo

      do i=1,surface%nsurface
         if (surface%x(i).lt.1.d-10) ln_work(i)=0
         if (surface%x(i).gt.1.d0-1.d-10) ln_work(i)=0
         if (surface%y(i).lt.1.d-10) ln_work(i)=0
         if (surface%y(i).gt.1.d0-1.d-10) ln_work(i)=0
      enddo

      do i=1,surface%nsurface
         if (ln_work(i).ne.0) then
            if (surface%spread_surface_points.eq.1) then
               surface%x(i)=points(1,i)/ln_work(i)
               surface%y(i)=points(2,i)/ln_work(i)
            elseif (surface%spread_surface_points.eq.2) then
               surface%x(i)=(pointsmin(1,i)+pointsmax(1,i))/2.d0
               surface%y(i)=(pointsmin(2,i)+pointsmax(2,i))/2.d0
            elseif (surface%spread_surface_points.eq.3) then
               points(1,i)=points(1,i)/ln_work(i)
               points(2,i)=points(2,i)/ln_work(i)
            endif
         endif
      enddo

      if (surface%spread_surface_points.eq.3) then
      allocate (ppoints(2,surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc ppoints in erode$')
      allocate (wpoints(surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc wpoints in erode$')
      wpoints=0.d0
      do i=1,surface%nt
         do k=1,3
            kp=mod(k,3)+1
            ic=surface%icon(k,i)
            icp=surface%icon(kp,i)
            wpoints(ic)=wpoints(ic)+sqrt((surface%x(icp)-points(1,ic))**2+(surface%y(icp)-points(2,ic))**2)*0.1d0
         enddo
      enddo
      ppoints=0.d0
      do i=1,surface%nt
         do k=1,3
            kp=mod(k,3)+1
            ic=surface%icon(k,i)
            icp=surface%icon(kp,i)
            ppoints(1,ic)=ppoints(1,ic)+sqrt((surface%x(icp)-points(1,ic))**2+(surface%y(icp)-points(2,ic))**2)*0.1d0*surface%x(icp)
            ppoints(2,ic)=ppoints(2,ic)+sqrt((surface%x(icp)-points(1,ic))**2+(surface%y(icp)-points(2,ic))**2)*0.1d0*surface%y(icp)
         enddo
      enddo
      do i=1,surface%nsurface
         if (ln_work(i).ne.0) then
             surface%x(i)=ppoints(1,i)/wpoints(i)
             surface%y(i)=ppoints(2,i)/wpoints(i)
         endif
      enddo
      deallocate (ppoints,wpoints)
      endif

      deallocate (points)
      deallocate (pointsmin)
      deallocate (pointsmax)
      deallocate (ln_work)

   endif

   do i=1,surface%nsurface
      call octree_interpolate (olsf%octree,olsf%noctree,olsf%icon, &
                               olsf%nleaves,olsf%lsf,olsf%nnode, &
                               surface%x(i),surface%y(i),surface%z(i),f)
      surface%z(i)=min(f,surface%z(i))
   enddo

   return

endif


! sedimentation


! change this to have a flag (0/1) so that current time has dt/2 added for
!minpoint call

zaggrade_current=zaggrade_init+aggrade_rate*(current_time+time_step)
x_prog1=x_prog_start+prog_rate_u*(current_time+time_step)
x_prog2=x_prog_end+prog_rate_u*(current_time+time_step)
!zaggrade_current2=zaggrade_init2+aggrade_rate*current_time
!if (iproc.eq.0) write(*,*) 'zaggrade_current: ', zaggrade_current

select case(sedimentation_type)

   case (1)
      i=1
      do i=1,surface%nsurface
!       if (iproc.eq.0) write(*,*) 'Surface elevation under consideration:', surface%z(i)
        if (surface%z(i).lt.zaggrade_current) then
!          if (iproc.eq.0) write(*,*) 'Elevation before adjustment:', surface%z(i)
           surface%z(i)=zaggrade_current
!          if (iproc.eq.0) write(*,*) 'Elevation after adjustment:', surface%z(i)
        endif
      enddo

   case (2)
      i=1
      do i=1,surface%nsurface
         if (surface%x(i).gt.x_agg_start.and.surface%x(i).lt.x_agg_end.and. surface%y(i).gt.y_agg_start.and.surface%y(i).lt.y_agg_end) then
            z_compare=zaggrade_current+x_agg_sinus_amp*(1+sin((surface%x(i)-x_agg_start)*3.14159/(x_agg_sinus_wavelth/2)-3.14159/2))*y_agg_sinus_amp*(1+sin((surface%y(i)-y_agg_start)*3.14159/(y_agg_sinus_wavelth/2)-3.14159/2))
            if (surface%z(i).lt.z_compare) then
            surface%z(i)=z_compare
            endif
         else
            if (surface%z(i).lt.zaggrade_current) then
               surface%z(i)=zaggrade_current
            endif
         endif
      enddo

   case (3)
      i=1
      do i=1,surface%nsurface
         if (surface%x(i).lt.x_prog1) then
            if (surface%z(i).lt.z_prog_init) then
               surface%z(i)=z_prog_init
            endif
         elseif (surface%x(i).gt.x_prog1.and.surface%x(i).lt.x_prog2) then
            z_compare=z_prog_fin+(z_prog_init-z_prog_fin)*exp(-((surface%x(i)-x_prog1)**2)/(x_prog_length**2) )
            if (surface%z(i).lt.z_compare) then
               surface%z(i)=z_compare
            endif
         elseif (surface%x(i).gt.x_prog2) then
            if (surface%z(i).lt.z_prog_fin) then
               surface%z(i)=z_prog_fin
            endif
         endif
      enddo

   case default
      if (iproc.eq.0) write(*,*) 'Sedimentation type ',sedimentation_type,' not recognized'

end select


! nn_setup (calculates delaunay triangles and other info)

ntmax=surface%nsurface*3
nhmax=surface%nsurface
npmax=surface%nsurface
nnpnmax=100
nmax=3*ntmax+npmax
nvmax=ntmax

allocate (points(2,surface%nsurface),stat=err); if (err.ne.0) call stop_run ('Error alloc points in erode$')
allocate (field(surface%nsurface),stat=err)   ; if (err.ne.0) call stop_run ('Error alloc field in erode$')
allocate (vertices(3,ntmax),stat=err)         ; if (err.ne.0) call stop_run ('Error alloc vertices in erode$')
allocate (centres(3,ntmax),stat=err)          ; if (err.ne.0) call stop_run ('Error alloc centress in erode$')
allocate (neighbour(3,ntmax),stat=err)        ; if (err.ne.0) call stop_run ('Error alloc neighbour in erode$')
allocate (hulltriangles(nhmax),stat=err)      ; if (err.ne.0) call stop_run ('Error alloc hulltriangles in erode$')
allocate (vis_tlist(nvmax),stat=err)          ; if (err.ne.0) call stop_run ('Error alloc vis_tlist in erode$')
allocate (vis_elist(nvmax),stat=err)          ; if (err.ne.0) call stop_run ('Error alloc vis_elist in erode$')
allocate (add_tlist(nvmax),stat=err)          ; if (err.ne.0) call stop_run ('Error alloc add_tlist in erode$')
allocate (ln_work(surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc ln_work in erode$')
allocate (lt_work_l(ntmax),stat=err)            ; if (err.ne.0) call stop_run ('Error alloc lt_work_l in erode$')
allocate (ln_work_l(surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc ln_work_l in erode$')

points(1,:)=surface%x
points(2,:)=surface%y
field(:)=surface%z
dmode=-2
nmode=0
clockwise=.true.
eps=1.d-20
eps=0.d0
      
call nn2d_setup (surface%nsurface,ntmax,nhmax,npmax,nnpnmax,nmax, &
                 points,dmode,nmode,clockwise,field,nt,vertices, &
                 centres,neighbour,nh,hulltriangles,nohalt_hull, &
                 loc,nnn,nnlist,ntrilist, &
                 eps,nvmax,vis_tlist,vis_elist,add_tlist, &
                 lt_work_l,ln_work_l)

allocate (work_d1(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d1 in erode$')
allocate (work_d2(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d2 in erode$')
allocate (work_d3(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d3 in erode$')
allocate (work_d4(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d4 in erode$')
allocate (work_d5(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d5 in erode$')
allocate (work_d6(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d6 in erode$')
allocate (work_d7(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_d7 in erode$')
allocate (work_i1(nnpnmax),stat=err)   ; if (err.ne.0) call stop_run ('Error alloc work_i1 in erode$')
allocate (work_i2(2,nnpnmax),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_i2 in erode$')
allocate (work_i3(nnpnmax),stat=err)   ; if (err.ne.0) call stop_run ('Error alloc work_i3 in erode$')
allocate (work_r1(nnpnmax,2),stat=err) ; if (err.ne.0) call stop_run ('Error alloc work_r1 in erode$')
allocate (work_r2(nnpnmax),stat=err)   ; if (err.ne.0) call stop_run ('Error alloc work_r2 in erode$')
allocate (lsf(olsf%nnode),stat=err)    ; if (err.ne.0) call stop_run ('Error alloc lsf in erode$')

nnext=.true.
int_method=3
loc=1

lsf=0.d0

do i=iproc+1,olsf%nnode,nproc
   x=olsf%x(i)
   y=olsf%y(i)
   call nn2D (x,y,points,vertices,neighbour,nnext,int_method,centres, &
              hulltriangles,nh,loc,field,nnpnmax,work_r1,work_r2, &
              work_d1,work_i1,work_i2,work_i3,work_d2,work_d3,work_d4, &
              work_d5,work_d6,work_d7,lt_work_l,ln_work_l,out,f,df,ddf,v)
   lsf(i)=f
enddo

olsf%lsf(1:olsf%nnode)=0.d0
call mpi_allreduce (lsf,olsf%lsf,olsf%nnode, &
                    mpi_double_precision,mpi_sum,mpi_comm_world,ierr)

! here spreads the points on the top surface to avoid collapse of the surface due to erosion
! for that it uses a barrycenter algorithm and a reinterpolation of the surface to avoid undesired diffusion effects

if (surface%spread_surface_points.gt.0) then

allocate (pointsmin(2,surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc pointsmin in erode$')
allocate (pointsmax(2,surface%nsurface),stat=err) ; if (err.ne.0) call stop_run ('Error alloc pointsmax in erode$')

points=0.d0
pointsmin=2.d0
pointsmax=-1.d0
ln_work=0
do i=1,surface%nt
  do k=1,3
  kp=mod(k,3)+1
  ic=surface%icon(k,i)
  icp=surface%icon(kp,i)
  points(1,ic)=points(1,ic)+surface%x(icp)
  points(2,ic)=points(2,ic)+surface%y(icp)
  pointsmin(1,ic)=min(pointsmin(1,ic),surface%x(icp))
  pointsmax(1,ic)=max(pointsmax(1,ic),surface%x(icp))
  pointsmin(2,ic)=min(pointsmin(2,ic),surface%y(icp))
  pointsmax(2,ic)=max(pointsmax(2,ic),surface%y(icp))
  ln_work(ic)=ln_work(ic)+1
  enddo
enddo
do i=1,surface%nsurface
if (surface%x(i).lt.1.d-10) ln_work(i)=0
if (surface%x(i).gt.1.d0-1.d-10) ln_work(i)=0
if (surface%y(i).lt.1.d-10) ln_work(i)=0
if (surface%y(i).gt.1.d0-1.d-10) ln_work(i)=0
enddo
do i=1,surface%nsurface
if (ln_work(i).ne.0) then
  if (surface%spread_surface_points.eq.1) then
  surface%x(i)=points(1,i)/ln_work(i)
  surface%y(i)=points(2,i)/ln_work(i)
  else
  surface%x(i)=(pointsmin(1,i)+pointsmax(1,i))/2.d0
  surface%y(i)=(pointsmin(2,i)+pointsmax(2,i))/2.d0
  endif
endif
enddo

deallocate (pointsmin,pointsmax)

endif

deallocate (points)
deallocate (field)
deallocate (vertices)
deallocate (centres)
deallocate (hulltriangles)
deallocate (vis_tlist)
deallocate (vis_elist)
deallocate (add_tlist)
deallocate (lt_work_l)
deallocate (ln_work)
deallocate (ln_work_l)
deallocate (work_d1)
deallocate (work_d2)
deallocate (work_d3)
deallocate (work_d4)
deallocate (work_d5)
deallocate (work_d6)
deallocate (work_d7)
deallocate (work_r1)
deallocate (work_r2)
deallocate (work_i1)
deallocate (work_i2)
deallocate (work_i3)
deallocate (lsf)

return
end
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
