!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              UPDATE_CLOUD_STRUCTURE    Feb. 2007                             |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine update_cloud_structure (cl,os,params,mat,ninject,nremove,istep)

!------------------------------------------------------------------------------|
!(((((((((((((((( Purpose of the routine  ))))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! this file contains the operations done on a 3D cloud of particles
! to check the density of particles, accordingly add or remove particles
! and transfer the fields carried by the cloud (here the original
! position of the particles)

! First it initializes the fields to the current (thus original) position

! then we also interpolate the fields (original position) onto the
! corresponding fields of the octree solve in order for the information 
! to be available during matrix building operation

! cl is the cloud
! os is the octree solve
! npmin is the minimum number of particles in any leaf of the octree solve
! npmax is the maximum number of particles in the smallest leaves of the
!    octree solve
! ninject is number of new particles injected in the cloud
! nremove is number of particles removed from the cloud
! levelmax_oct is maximum level of octree solve

!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|
      
use definitions
!use mpi

implicit none

include 'mpif.h'

type (cloud) cl
type (octreesolve) os
type (parameters) params
type (material) mat(0:params%nmat)
integer ninject
integer nremove
type (octreev) ov
integer istep

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer ierr,iproc,nproc,i,j,k,ic,jc,kk,ii,xi,yi,zi
integer np,ne,nn,npnew,ie,npswap,ntest,ninjectp,kinject,kkinject,nchoice
integer,dimension(:),allocatable::loc,nloc,ntot,remove,lev,done
integer err,leaf,level,locc,ndist,nisolated,ntodo,nremovep,icut,icutlsf
integer, dimension(:,:), allocatable :: imemswap

double precision x0,y0,z0,dxyz,distmax,dist,distmin,xx,yy,zz,h(8),ww,r,s,t,prod
double precision lsftemp,eps
double precision,dimension(:),allocatable::xtest,ytest,ztest
double precision,dimension(:),allocatable::xinject,yinject,zinject
double precision,dimension(:),allocatable::straininject,e2dpinject
double precision,dimension(:),allocatable::x0inject,y0inject,z0inject
double precision,dimension(:),allocatable::x0mpinject,y0mpinject,z0mpinject
double precision,dimension(:),allocatable::strain,w,e2dp
double precision,dimension(:,:),allocatable::memswap

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

eps = 1.d-10

np=cl%np
ne=os%nleaves
nn=os%nnode
ntest=10

! check number of particles to inject/remove to adjust the size of arrays
! in main program

allocate (ntot(ne),stat=err) ; if (err.ne.0) call stop_run ('error in allocating ntot in update_cloud$')
allocate (lev(ne),stat=err) ; if (err.ne.0) call stop_run ('error in allocating lev in update_cloud$')

ntot=0
do i=1,np
   call octree_find_leaf (os%octree,os%noctree,cl%x(i),cl%y(i),cl%z(i), &
                          leaf,level,locc,x0,y0,z0,dxyz)
   ntot(leaf)=ntot(leaf)+1
enddo

call octree_find_element_level (os%octree,os%noctree,lev,ne)

ninject=0
nremove=0
! Inject using regularly spaced grid for initial cloud
if (params%gridcloud .and. cl%np.eq.0) then
  ninject=params%clnpx*params%clnpy*params%clnpz*ne
  if (params%centerptcloud .and. (params%clnpx*params%clnpy*params%clnpz) .ne. 1) ninject=ninject+ne
! Inject up to npmin for each element, but limit to npmax
else
  do ie=1,ne
    if (ntot(ie).lt.params%npmin) ninject=ninject+(params%npmin-ntot(ie))
    if (ntot(ie).gt.params%npmax .and. lev(ie).ge.params%levelmax_oct) &
       nremove=nremove+(ntot(ie)-params%npmax)
  enddo
endif

deallocate (ntot)
deallocate (lev)

if (ninject.gt.0.and.iproc.eq.0) write (8,*) 'Injecting ',ninject,' particles'
if (nremove.gt.0.and.iproc.eq.0) write (8,*) 'Removing ',nremove,' particles'

npswap=cl%np
if (npswap.ne.0) then
   allocate (memswap(14,npswap),stat=err) ; if (err.ne.0) call stop_run ('error in alloc memswap in main$')
   allocate (imemswap(3,npswap),stat=err) ; if (err.ne.0) call stop_run ('error in alloc imemswap in main$')
   memswap(1,:)=cl%x
   memswap(2,:)=cl%y
   memswap(3,:)=cl%z
   memswap(4,:)=cl%x0
   memswap(5,:)=cl%y0
   memswap(6,:)=cl%z0
   memswap(7,:)=cl%x0mp
   memswap(8,:)=cl%y0mp
   memswap(9,:)=cl%z0mp
   memswap(10,:)=cl%strain
   memswap(11,:)=cl%lsf0
   memswap(12,:)=cl%temp
   memswap(13,:)=cl%press
   memswap(14,:)=cl%e2dp
   imemswap(1,:)=cl%tag
   imemswap(2,:)=cl%matnum
   imemswap(3,:)=cl%ematnump
endif

deallocate (cl%x)
deallocate (cl%y)
deallocate (cl%z)
deallocate (cl%x0)
deallocate (cl%y0)
deallocate (cl%z0)
deallocate (cl%x0mp)
deallocate (cl%y0mp)
deallocate (cl%z0mp)
deallocate (cl%strain)
deallocate (cl%temp)
deallocate (cl%press)
deallocate (cl%lsf0)
deallocate (cl%e2dp)
deallocate (cl%tag)
deallocate (cl%matnum)
deallocate (cl%ematnump)

npnew=cl%np+ninject-nremove
npnew=max(npnew,npswap)

allocate (cl%x(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%x in main$')
allocate (cl%y(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%y in main$')
allocate (cl%z(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%z in main$')
allocate (cl%x0(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%x0 in main$')
allocate (cl%y0(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%y0 in main$')
allocate (cl%z0(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%z0 in main$')
allocate (cl%x0mp(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%x0mp in main$')
allocate (cl%y0mp(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%y0mp in main$')
allocate (cl%z0mp(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%z0mp in main$')
allocate (cl%strain(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%strain in main$')
allocate (cl%temp(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%temp in main$')
allocate (cl%press(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%press in main$')
allocate (cl%lsf0(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%lsf0 in main$')
allocate (cl%e2dp(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%e2dp in main$')
allocate (cl%tag(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%tag in main$')
allocate (cl%matnum(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%matnum in main$')
allocate (cl%ematnump(npnew),stat=err) ; if (err.ne.0) call stop_run ('error in alloc cl%ematnump in main$')
        
if (npswap.ne.0) then
   cl%x(1:npswap)=memswap(1,:)
   cl%y(1:npswap)=memswap(2,:)
   cl%z(1:npswap)=memswap(3,:)
   cl%x0(1:npswap)=memswap(4,:)
   cl%y0(1:npswap)=memswap(5,:)
   cl%z0(1:npswap)=memswap(6,:)
   cl%x0mp(1:npswap)=memswap(7,:)
   cl%y0mp(1:npswap)=memswap(8,:)
   cl%z0mp(1:npswap)=memswap(9,:)
   cl%strain(1:npswap)=memswap(10,:)
   cl%lsf0(1:npswap)=memswap(11,:)
   cl%temp(1:npswap)=memswap(12,:)
   cl%press(1:npswap)=memswap(13,:)
   cl%e2dp(1:npswap)=memswap(14,:)
   cl%tag(1:npswap)=imemswap(1,:)
   cl%matnum(1:npswap)=imemswap(2,:)
   cl%ematnump(1:npswap)=imemswap(3,:)
   deallocate (memswap)
   deallocate (imemswap)
else
   cl%strain=0.d0
   cl%lsf0=0.d0
   cl%temp=0.d0
   cl%press=0.d0
   cl%e2dp=0.d0
   cl%tag=1
   cl%matnum=0
   cl%ematnump=0
endif

! first build location array
! ntot(ie) is number of particles in leaf ie
! nloc(ie) is location in array loc of where the particles belonging to ie are stored
! loc(nloc(ie):nloc(ie)+ntot(ie)-1) is list of particles belonging to ie

allocate (loc(np),stat=err) ; if (err.ne.0) call stop_run ('error in alloc loc in update_cloud$')
allocate (nloc(ne),stat=err) ; if (err.ne.0) call stop_run ('error in alloc nloc in update_cloud$')
allocate (ntot(ne),stat=err) ; if (err.ne.0) call stop_run ('error in alloc ntot in update_cloud$')
allocate (lev(ne),stat=err) ; if (err.ne.0) call stop_run ('error in alloc lev in update_cloud$')
allocate (remove(np),stat=err) ; if (err.ne.0) call stop_run ('error in alloc remove in update_cloud$')

ntot=0
do i=1,np
   call octree_find_leaf (os%octree,os%noctree,cl%x(i),cl%y(i),cl%z(i), &
                          leaf,level,locc,x0,y0,z0,dxyz)
   ntot(leaf)=ntot(leaf)+1
enddo

nloc(1)=1
do ie=2,ne
   nloc(ie)=nloc(ie-1)+ntot(ie-1)
enddo

ntot=0
loc=0
do i=1,np
   call octree_find_leaf (os%octree,os%noctree,cl%x(i),cl%y(i),cl%z(i), &
                          leaf,level,locc,x0,y0,z0,dxyz)
   loc(nloc(leaf)+ntot(leaf))=i
   ntot(leaf)=ntot(leaf)+1
enddo

call octree_find_element_level (os%octree,os%noctree,lev,ne)

! first check for injection

ninjectp=ninject
! Inject using regularly spaced initial cloud, if desired
if (params%gridcloud .and. cl%np.eq.0) then
  ninject=params%clnpx*params%clnpy*params%clnpz*ne
  if (params%centerptcloud .and. (params%clnpx*params%clnpy*params%clnpz) .ne. 1) ninject=ninject+ne
! Inject up to npmin in each element
else
  ninject=0
  do ie=1,ne
     if (ntot(ie).lt.params%npmin) then
        ninject=ninject+(params%npmin-ntot(ie))
     endif
  enddo
endif

allocate (xinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc xinject in update_cloud$')
allocate (yinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc yinject in update_cloud$')
allocate (zinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc zinject in update_cloud$')
allocate (straininject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc straininject in update_cloud$')
allocate (x0inject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc x0inject in update_cloud$')
allocate (y0inject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc y0inject in update_cloud$')
allocate (z0inject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc z0inject in update_cloud$')
allocate (x0mpinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc x0mpinject in update_cloud$')
allocate (y0mpinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc y0mpinject in update_cloud$')
allocate (z0mpinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc z0mpinject in update_cloud$')
allocate (e2dpinject(ninject),stat=err) ; if (err.ne.0) call stop_run ('error in alloc e2dpinject in update_cloud$')

if (ninject.ne.ninjectp) call stop_run ('error 1 in update_cloud$')

kinject=0
! Inject using regularly spaced initial cloud, if desired
if (params%gridcloud .and. cl%np.eq.0) then
  do ie=1,ne
    do zi=1,params%clnpz
      do yi=1,params%clnpy
        do xi=1,params%clnpx
          kinject=kinject+1
          xinject(kinject)=minval(os%x(os%icon(:,ie))) + 1.d0/params%clnpx * (0.5d0 + (xi-1)) * 1.d0/(2.d0**lev(ie))
          yinject(kinject)=minval(os%y(os%icon(:,ie))) + 1.d0/params%clnpy * (0.5d0 + (yi-1)) * 1.d0/(2.d0**lev(ie))
          zinject(kinject)=minval(os%z(os%icon(:,ie))) + 1.d0/params%clnpz * (0.5d0 + (zi-1)) * 1.d0/(2.d0**lev(ie))
        enddo
      enddo
    enddo
    if (params%centerptcloud .and. (params%clnpx*params%clnpy*params%clnpz).ne.1) then
      kinject=kinject+1
      xinject(kinject)=minval(os%x(os%icon(:,ie))) + 0.5d0/(2**lev(ie))
      yinject(kinject)=minval(os%y(os%icon(:,ie))) + 0.5d0/(2**lev(ie))
      zinject(kinject)=minval(os%z(os%icon(:,ie))) + 0.5d0/(2**lev(ie))
    endif  
  enddo
! Inject up to npmin in each element
else
  do ie=1,ne
    if (ntot(ie).lt.params%npmin) then
      kkinject=kinject+1
      do ii=1,params%npmin-ntot(ie)
        allocate (xtest(ntest),stat=err) ; if (err.ne.0) call stop_run ('error in alloc xtest in update_cloud$')
        allocate (ytest(ntest),stat=err) ; if (err.ne.0) call stop_run ('error in alloc ytest in update_cloud$')
        allocate (ztest(ntest),stat=err) ; if (err.ne.0) call stop_run ('error in alloc ztest in update_cloud$')
        call random_number (xtest)
        call random_number (ytest)
        call random_number (ztest)
        xtest=os%x(os%icon(1,ie))+xtest*(os%x(os%icon(2,ie))-os%x(os%icon(1,ie)))
        ytest=os%y(os%icon(1,ie))+ytest*(os%y(os%icon(3,ie))-os%y(os%icon(1,ie)))
        ztest=os%z(os%icon(1,ie))+ztest*(os%z(os%icon(5,ie))-os%z(os%icon(1,ie)))
        distmax=0.d0
        nchoice=0
        do i=1,ntest
          dist=0.d0
          ndist=0
          do k=1,ntot(ie)
            dist=dist+(xtest(i)-cl%x(loc(nloc(ie)+k-1)))**2+ &
                      (ytest(i)-cl%y(loc(nloc(ie)+k-1)))**2+ &
                      (ztest(i)-cl%z(loc(nloc(ie)+k-1)))**2
            ndist=ndist+1
          enddo
          do k=1,8
            dist=dist+(xtest(i)-os%x(os%icon(k,ie)))**2+ &
                      (ytest(i)-os%y(os%icon(k,ie)))**2+ &
                      (ztest(i)-os%z(os%icon(k,ie)))**2
            ndist=ndist+1
          enddo
          do k=kkinject,kinject
            dist=dist+(xtest(i)-xinject(k))**2+ &
                      (ytest(i)-yinject(k))**2+ &
                      (ztest(i)-zinject(k))**2
            ndist=ndist+1
          enddo
          dist=dist/ndist
          if (dist.gt.distmax) then
            distmax=dist
            nchoice=i
          endif
        enddo
        if (nchoice.eq.0) call stop_run ('error in finding inject part loc$')
        ! found the random particle that is the furthest away from the others and the
        ! 8 nodes of the leaf and previously injected ones; now we inject a particle
        kinject=kinject+1
        xinject(kinject)=xtest(nchoice)
        yinject(kinject)=ytest(nchoice)
        zinject(kinject)=ztest(nchoice)
        deallocate (xtest)
        deallocate (ytest)
        deallocate (ztest)
      enddo
    endif
  enddo
endif

if (kinject.ne.ninjectp) call stop_run ('error 2 in update_cloud$')

! second check for removal

remove=0
do ie=1,ne
   if (ntot(ie).gt.params%npmax.and.lev(ie).ge.params%levelmax_oct) then
      do ii=1,ntot(ie)-params%npmax
         distmin=1.d2
         nchoice=0
         do i=1,ntot(ie)
            if (remove(loc(nloc(ie)+i-1)).eq.0) then
               dist=0.d0
               ndist=0
               xx=cl%x(loc(nloc(ie)+i-1))
               yy=cl%y(loc(nloc(ie)+i-1))
               zz=cl%z(loc(nloc(ie)+i-1))
               do k=1,ntot(ie)
                  if (remove(loc(nloc(ie)+k-1)).eq.0 .and. k.ne.i) then
                     dist=dist+(xx-cl%x(loc(nloc(ie)+k-1)))**2+ &
                               (yy-cl%y(loc(nloc(ie)+k-1)))**2+ &
                               (zz-cl%z(loc(nloc(ie)+k-1)))**2
                     ndist=ndist+1
                  endif
               enddo
               do k=1,8
                  dist=dist+(xx-os%x(os%icon(k,ie)))**2+ &
                            (yy-os%y(os%icon(k,ie)))**2+ &
                            (zz-os%z(os%icon(k,ie)))**2
                  ndist=ndist+1
               enddo
               dist=dist/ndist
               if (dist.lt.distmin) then
                  distmin=dist
                  nchoice=loc(nloc(ie)+i-1)
               endif
            endif
         enddo
         if (nchoice.eq.0) call stop_run ('error in finding inject part loc$')
         ! found the particle that is the closest to the others and the
         ! 8 nodes of the leaf; we remove it
         remove(nchoice)=1
      enddo
   endif
enddo

! now interpolate onto the nodes of the octree solve

os%strain=0.d0
os%e2dp=0.d0
allocate (strain(nn),stat=err) ; if (err.ne.0) call stop_run ('error in alloc strain in update_cloud$')
allocate (e2dp(nn),stat=err) ; if (err.ne.0) call stop_run ('error in alloc e2dp in update_cloud$')
allocate (w(nn),stat=err) ; if (err.ne.0) call stop_run ('error in alloc w in update_cloud')

strain=0.d0
e2dp=0.d0
w=0.d0
do i=1,np
   call octree_find_leaf (os%octree,os%noctree,cl%x(i),cl%y(i),cl%z(i), &
                          leaf,level,locc,x0,y0,z0,dxyz)
   r=(cl%x(i)-x0)/dxyz*2.d0-1.d0
   s=(cl%y(i)-y0)/dxyz*2.d0-1.d0
   t=(cl%z(i)-z0)/dxyz*2.d0-1.d0
   h(1)=(1.d0-r)*(1.d0-s)*(1.d0-t)/8.d0
   h(2)=(1.d0+r)*(1.d0-s)*(1.d0-t)/8.d0
   h(3)=(1.d0-r)*(1.d0+s)*(1.d0-t)/8.d0
   h(4)=(1.d0+r)*(1.d0+s)*(1.d0-t)/8.d0
   h(5)=(1.d0-r)*(1.d0-s)*(1.d0+t)/8.d0
   h(6)=(1.d0+r)*(1.d0-s)*(1.d0+t)/8.d0
   h(7)=(1.d0-r)*(1.d0+s)*(1.d0+t)/8.d0
   h(8)=(1.d0+r)*(1.d0+s)*(1.d0+t)/8.d0
   do k=1,8
      ic=os%icon(k,leaf)
      ww=h(k)/dxyz**3
      w(ic)=w(ic)+ww
      strain(ic)=strain(ic)+ww*cl%strain(i)
      e2dp(ic)=e2dp(ic)+ww*cl%e2dp(i)
   enddo
enddo

if (np.gt.0) then
   allocate (done(os%nnode),stat=err) ; if (err.ne.0) call stop_run ('Error alloc done in update_cloud$')
   nisolated=0
   done=0
   do i=1,os%nnode
      if (w(i).gt.0.d0) then
         os%strain(i)=strain(i)/w(i)
         os%e2dp(i)=e2dp(i)/w(i)
      else
         done(i)=1
         nisolated=nisolated+1
      endif
   enddo
   if (iproc.eq.0) write (8,*) 'There were ',nisolated,' octree nodes'

111   continue

   strain=0.d0
   e2dp=0.d0
   w=0.d0
   do ie=1,os%nleaves
      do k=1,8
         ic=os%icon(k,ie)
         if (done(ic).eq.1) then
            do kk=1,8
               jc=os%icon(kk,ie)
               if (done(jc).eq.0) then
                  ww=1.d0/dxyz**3
                  w(ic)=w(ic)+ww
                  strain(ic)=strain(ic)+os%strain(jc)*ww
                  e2dp(ic)=e2dp(ic)+os%e2dp(jc)*ww
               endif
            enddo
         endif
      enddo
   enddo

   ntodo=0
   do i=1,os%nnode
      if (done(i).eq.1) then
         if (w(i).gt.0.d0) then
            done(i)=0
            os%strain(i)=strain(i)/w(i)
            os%e2dp(i)=e2dp(i)/w(i)
         else
            ntodo=ntodo+1
         endif
      endif
   enddo

   if (iproc.eq.0) write (8,*) ntodo,' Isolated left '

   if (ntodo.ne.0) goto 111

   deallocate (done)

endif

deallocate (strain)
deallocate (e2dp)
deallocate (w)

! now calculate the field value at the new (injected points)

straininject=0.d0
e2dpinject=0.d0
x0inject=0.d0
y0inject=0.d0
z0inject=0.d0
x0mpinject=0.d0
y0mpinject=0.d0
z0mpinject=0.d0
do i=1,ninject
   call octree_find_leaf (os%octree,os%noctree,xinject(i),yinject(i),zinject(i), &
                          leaf,level,locc,x0,y0,z0,dxyz)
   r=(xinject(i)-x0)/dxyz*2.d0-1.d0
   s=(yinject(i)-y0)/dxyz*2.d0-1.d0
   t=(zinject(i)-z0)/dxyz*2.d0-1.d0
   h(1)=(1.d0-r)*(1.d0-s)*(1.d0-t)/8.d0
   h(2)=(1.d0+r)*(1.d0-s)*(1.d0-t)/8.d0
   h(3)=(1.d0-r)*(1.d0+s)*(1.d0-t)/8.d0
   h(4)=(1.d0+r)*(1.d0+s)*(1.d0-t)/8.d0
   h(5)=(1.d0-r)*(1.d0-s)*(1.d0+t)/8.d0
   h(6)=(1.d0+r)*(1.d0-s)*(1.d0+t)/8.d0
   h(7)=(1.d0-r)*(1.d0+s)*(1.d0+t)/8.d0
   h(8)=(1.d0+r)*(1.d0+s)*(1.d0+t)/8.d0
   do k=1,8
      straininject(i)=straininject(i)+os%strain(os%icon(k,leaf))*h(k)
      e2dpinject(i)=e2dpinject(i)+os%e2dp(os%icon(k,leaf))*h(k)
      x0inject(i)=x0inject(i)+os%x(os%icon(k,leaf))*h(k)
      y0inject(i)=y0inject(i)+os%y(os%icon(k,leaf))*h(k)
      z0inject(i)=z0inject(i)+os%z(os%icon(k,leaf))*h(k)
      x0mpinject(i)=x0mpinject(i)+os%x(os%icon(k,leaf))*h(k)
      y0mpinject(i)=y0mpinject(i)+os%y(os%icon(k,leaf))*h(k)
      z0mpinject(i)=z0mpinject(i)+os%z(os%icon(k,leaf))*h(k)
   enddo
enddo

! modify the particle cloud for removal/injection of particles

nremovep=0
do i=1,np
   if (remove(i).eq.1) then
      nremovep=nremovep+1
   else
      cl%x(i-nremovep)=cl%x(i)
      cl%y(i-nremovep)=cl%y(i)
      cl%z(i-nremovep)=cl%z(i)
      cl%x0(i-nremovep)=cl%x0(i)
      cl%y0(i-nremovep)=cl%y0(i)
      cl%z0(i-nremovep)=cl%z0(i)
      cl%x0mp(i-nremovep)=cl%x0mp(i)
      cl%y0mp(i-nremovep)=cl%y0mp(i)
      cl%z0mp(i-nremovep)=cl%z0mp(i)
      cl%strain(i-nremovep)=cl%strain(i)
      cl%lsf0(i-nremovep)=cl%lsf0(i)
      cl%temp(i-nremovep)=cl%temp(i)
      cl%press(i-nremovep)=cl%press(i)
      cl%e2dp(i-nremovep)=cl%e2dp(i)
      cl%tag(i-nremovep)=cl%tag(i)
      cl%matnum(i-nremovep)=cl%matnum(i)
      cl%ematnump(i-nremovep)=cl%ematnump(i)
   endif
enddo

if (nremovep.ne.nremove) call stop_run ('error in update_cloud/remove$')

np=np-nremovep

do i=1,ninject
   np=np+1
   cl%x(np)=xinject(i)
   cl%y(np)=yinject(i)
   cl%z(np)=zinject(i)
   cl%x0(np)=x0inject(i)
   cl%y0(np)=y0inject(i)
   cl%z0(np)=z0inject(i)
   cl%x0mp(np)=x0mpinject(i)
   cl%y0mp(np)=y0mpinject(i)
   cl%z0mp(np)=z0mpinject(i)
   cl%strain(np)=straininject(i)
   cl%lsf0(np)=0.d0
   cl%temp(np)=0.d0
   cl%press(np)=0.d0
   cl%e2dp(np)=e2dpinject(i)
   cl%ntag=cl%ntag+1
   cl%tag(np)=cl%ntag
   cl%matnum(np)=0
   cl%ematnump(np)=0
enddo

! Assign material number to injected cloud particles using LSFs or elemental
! material number
if (cl%np > 0) then
  do i=np-ninject+1,np
    icut=0
    call octree_find_leaf (os%octree,os%noctree,cl%x(i),cl%y(i),cl%z(i),leaf,  &
                          level,locc,x0,y0,z0,dxyz)
    do j=1,os%nlsf
      icutlsf=0
      prod=os%lsf(os%icon(1,leaf),j)
      do k=2,params%mpe
        if (prod*os%lsf(os%icon(k,leaf),j) <= -eps .and. icutlsf==0) then
          icutlsf=1
          cl%matnum(i)=params%materialn(j-1)
          lsftemp=0.d0
          r=(cl%x(i)-x0)/dxyz*2.d0-1.d0
          s=(cl%y(i)-y0)/dxyz*2.d0-1.d0
          t=(cl%z(i)-z0)/dxyz*2.d0-1.d0
          h(1)=(1.d0-r)*(1.d0-s)*(1.d0-t)/8.d0
          h(2)=(1.d0+r)*(1.d0-s)*(1.d0-t)/8.d0
          h(3)=(1.d0-r)*(1.d0+s)*(1.d0-t)/8.d0
          h(4)=(1.d0+r)*(1.d0+s)*(1.d0-t)/8.d0
          h(5)=(1.d0-r)*(1.d0-s)*(1.d0+t)/8.d0
          h(6)=(1.d0+r)*(1.d0-s)*(1.d0+t)/8.d0
          h(7)=(1.d0-r)*(1.d0+s)*(1.d0+t)/8.d0
          h(8)=(1.d0+r)*(1.d0+s)*(1.d0+t)/8.d0
          do kk=1,8
            lsftemp=lsftemp+os%lsf(os%icon(kk,leaf),j)*h(kk)
          enddo
          if (lsftemp <= -eps) cl%matnum(i)=params%materialn(j)
        endif
      enddo
      if (icutlsf /= 0) icut=1
    enddo
    ! Original version below
    if (icut == 0) then
      cl%matnum(i)=os%matnum(leaf)
      !write (*,*) 'Injecting particle into leaf ',leaf,' with matnum ',cl%matnum(i)
    endif
    !if (icut == 0) cl%matnum(i)=1

    ! Check for material transitions
    call mat_trans (params,mat,cl%matnum(i),cl%x(i),cl%y(i),cl%z(i))
  enddo
endif

! in first step, we set the original particle positions to their
! current position

if (cl%np.eq.0) then
   do i=1,np
      cl%x0(i)=cl%x(i)
      cl%y0(i)=cl%y(i)
      cl%z0(i)=cl%z(i)
      cl%x0mp(i)=cl%x(i)
      cl%y0mp(i)=cl%y(i)
      cl%z0mp(i)=cl%z(i)
      cl%strain(i)=0.d0
      cl%temp(i)=0.d0
      cl%press(i)=0.d0
      cl%lsf0(i)=0.d0
      cl%e2dp(i)=0.d0
      cl%tag(i)=i
      cl%matnum(i)=0
      cl%ematnump(i)=0
   enddo
   cl%ntag=np
endif

cl%np=np

deallocate (xinject)
deallocate (yinject)
deallocate (zinject)
deallocate (straininject)
deallocate (e2dpinject)
deallocate (x0inject,y0inject,z0inject)
deallocate (x0mpinject,y0mpinject,z0mpinject)
deallocate (loc)
deallocate (nloc)
deallocate (ntot)
deallocate (remove)
deallocate (lev)

if (iproc.eq.0) then
   write (8,*) 'Strain on osolve ',minval(os%strain),maxval(os%strain)
   write (8,*) cl%np,' particles in 3D cloud'
end if

return
end
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
